@@ -1,11 +1,11 @@
 use crate::utils::{dag, info, Error, Result, INTERNAL_ERR};
 use cargo_metadata::Metadata;
-use clap::{AppSettings, Parser};
+use clap::Parser;
 use std::process::Command;
 
 /// Execute an arbitrary command in each crate
 #[derive(Debug, Parser)]
-#[clap(setting = AppSettings::TrailingVarArg)]
+#[clap(trailing_var_arg(true))]
 pub struct Exec {
     /// Continue executing command despite non-zero exit in a given crate
     #[clap(long)]
@@ -31,7 +31,7 @@ impl Exec {
             let dir = pkg
                 .manifest_path
                 .parent()
-                .ok_or(Error::ManifestHasNoParent(pkg.name.clone()))?;
+                .ok_or_else(|| Error::ManifestHasNoParent(pkg.name.clone()))?;
 
             let status = Command::new(self.args.get(0).expect(INTERNAL_ERR))
                 .args(&self.args[1..])
