@@ -1,12 +1,22 @@
-use crate::utils::{debug, info, validate_value_containing_name, Error, INTERNAL_ERR};
-use clap::{ArgSettings, Parser};
+use crate::utils::{
+    debug, info, validate_value_containing_name, Error, WorkspaceConfig, INTERNAL_ERR,
+};
+
+use camino::Utf8PathBuf;
+use clap::Parser;
 use glob::Pattern;
 use semver::Version;
-use std::collections::BTreeMap as Map;
-use std::{path::PathBuf, process::Command};
 
-pub fn git<'a>(root: &PathBuf, args: &[&'a str]) -> Result<(String, String), Error> {
-    debug!("git", args.clone().join(" "));
+use std::{
+    collections::BTreeMap as Map,
+    process::{Command, ExitStatus},
+};
+
+pub fn git<'a>(
+    root: &Utf8PathBuf,
+    args: &[&'a str],
+) -> Result<(ExitStatus, String, String), Error> {
+    debug!("git", args.to_vec().join(" "));
 
     let output = Command::new("git")
         .current_dir(root)
@@ -18,6 +28,7 @@ pub fn git<'a>(root: &PathBuf, args: &[&'a str]) -> Result<(String, String), Err
         })?;
 
     Ok((
+        output.status,
         String::from_utf8(output.stdout)?.trim().to_owned(),
         String::from_utf8(output.stderr)?.trim().to_owned(),
     ))
@@ -33,14 +44,9 @@ pub struct GitOpt {
     ])]
     pub no_git_commit: bool,
 
-    /// Specify which branches to allow from
-    #[clap(
-        long,
-        default_value = "master",
-        value_name = "pattern",
-        setting(ArgSettings::ForbidEmptyValues)
-    )]
-    pub allow_branch: String,
+    /// Specify which branches to allow from [default: master]
+    #[clap(long, value_name = "pattern", forbid_empty_values(true))]
+    pub allow_branch: Option<String>,
 
     /// Amend the existing commit, instead of generating a new one
     #[clap(long)]
@@ -51,7 +57,7 @@ pub struct GitOpt {
         short,
         long,
         conflicts_with_all = &["amend"],
-        setting(ArgSettings::ForbidEmptyValues)
+        forbid_empty_values(true)
     )]
     pub message: Option<String>,
 
@@ -77,7 +83,7 @@ pub struct GitOpt {
         default_value = "%n@",
         value_name = "prefix",
         validator = validate_value_containing_name,
-        setting(ArgSettings::ForbidEmptyValues)
+        forbid_empty_values(true)
     )]
     pub individual_tag_prefix: String,
 
@@ -90,17 +96,21 @@ pub struct GitOpt {
         long,
         default_value = "origin",
         value_name = "remote",
-        setting(ArgSettings::ForbidEmptyValues)
+        forbid_empty_values(true)
     )]
     pub git_remote: String,
 }
 
 impl GitOpt {
-    pub fn validate(&self, root: &PathBuf) -> Result<Option<String>, Error> {
+    pub fn validate(
+        &self,
+        root: &Utf8PathBuf,
+        config: &WorkspaceConfig,
+    ) -> Result<Option<String>, Error> {
         let mut ret = None;
 
         if !self.no_git_commit {
-            let (out, err) = git(root, &["rev-list", "--count", "--all", "--max-count=1"])?;
+            let (_, out, err) = git(root, &["rev-list", "--count", "--all", "--max-count=1"])?;
 
             if err.contains("not a git repository") {
                 return Err(Error::NotGit);
@@ -110,7 +120,7 @@ impl GitOpt {
                 return Err(Error::NoCommits);
             }
 
-            let (branch, _) = git(root, &["rev-parse", "--abbrev-ref", "HEAD"])?;
+            let (_, branch, _) = git(root, &["rev-parse", "--abbrev-ref", "HEAD"])?;
 
             if branch == "HEAD" {
                 return Err(Error::NotBranch);
@@ -118,14 +128,23 @@ impl GitOpt {
 
             ret = Some(branch.clone());
 
+            // Get the final `allow_branch` value
+            let allow_branch_default_value = String::from("master");
+            let allow_branch = self.allow_branch.as_ref().unwrap_or_else(|| {
+                config
+                    .allow_branch
+                    .as_ref()
+                    .unwrap_or(&allow_branch_default_value)
+            });
+
             // Treat `main` as `master`
-            let test_branch = if branch == "main" && self.allow_branch == "master" {
+            let test_branch = if branch == "main" && allow_branch.as_str() == "master" {
                 "master".into()
             } else {
                 branch.clone()
             };
 
-            let pattern = Pattern::new(&self.allow_branch)?;
+            let pattern = Pattern::new(&allow_branch)?;
 
             if !pattern.matches(&test_branch) {
                 return Err(Error::BranchNotAllowed {
@@ -137,7 +156,7 @@ impl GitOpt {
             if !self.no_git_push {
                 let remote_branch = format!("{}/{}", self.git_remote, branch);
 
-                let (out, _) = git(
+                let (_, out, _) = git(
                     root,
                     &[
                         "show-ref",
@@ -155,7 +174,7 @@ impl GitOpt {
 
                 git(root, &["remote", "update"])?;
 
-                let (out, _) = git(
+                let (_, out, _) = git(
                     root,
                     &[
                         "rev-list",
@@ -179,10 +198,11 @@ impl GitOpt {
 
     pub fn commit(
         &self,
-        root: &PathBuf,
+        root: &Utf8PathBuf,
         new_version: &Option<Version>,
         new_versions: &Map<String, Version>,
         branch: Option<String>,
+        config: &WorkspaceConfig,
     ) -> Result<(), Error> {
         if !self.no_git_commit {
             info!("version", "committing changes");
@@ -190,8 +210,8 @@ impl GitOpt {
             let branch = branch.expect(INTERNAL_ERR);
             let added = git(root, &["add", "-u"])?;
 
-            if !added.0.is_empty() || !added.1.is_empty() {
-                return Err(Error::NotAdded(added.0, added.1));
+            if !added.0.success() {
+                return Err(Error::NotAdded(added.1, added.2));
             }
 
             let mut args = vec!["commit".to_string()];
@@ -222,8 +242,8 @@ impl GitOpt {
 
             let committed = git(root, &args.iter().map(|x| x.as_str()).collect::<Vec<_>>())?;
 
-            if !committed.0.contains(&branch) || !committed.1.is_empty() {
-                return Err(Error::NotCommitted(committed.0, committed.1));
+            if !committed.0.success() {
+                return Err(Error::NotCommitted(committed.1, committed.2));
             }
 
             if !self.no_git_tag {
@@ -236,7 +256,7 @@ impl GitOpt {
                     }
                 }
 
-                if !self.no_individual_tags {
+                if !(self.no_individual_tags || config.no_individual_tags.unwrap_or_default()) {
                     for (p, v) in new_versions {
                         let tag = format!("{}{}", self.individual_tag_prefix.replace("%n", p), v);
                         self.tag(root, &tag, &tag)?;
@@ -249,8 +269,8 @@ impl GitOpt {
 
                 let pushed = git(root, &["push", "--follow-tags", &self.git_remote, &branch])?;
 
-                if !pushed.0.is_empty() || !pushed.1.starts_with("To") {
-                    return Err(Error::NotPushed(pushed.0, pushed.1));
+                if !pushed.0.success() {
+                    return Err(Error::NotPushed(pushed.1, pushed.2));
                 }
             }
         }
@@ -258,11 +278,11 @@ impl GitOpt {
         Ok(())
     }
 
-    fn tag(&self, root: &PathBuf, tag: &str, msg: &str) -> Result<(), Error> {
-        let tagged = git(root, &["tag", &tag, "-m", &msg])?;
+    fn tag(&self, root: &Utf8PathBuf, tag: &str, msg: &str) -> Result<(), Error> {
+        let tagged = git(root, &["tag", tag, "-m", msg])?;
 
-        if !tagged.0.is_empty() || !tagged.1.is_empty() {
-            return Err(Error::NotTagged(tag.to_string(), tagged.0, tagged.1));
+        if !tagged.0.success() {
+            return Err(Error::NotTagged(tag.to_string(), tagged.1, tagged.2));
         }
 
         Ok(())
