@@ -1,9 +1,10 @@
 use crate::utils::{
-    cargo, change_versions, info, ChangeData, ChangeOpt, Error, GitOpt, Pkg, Result, INTERNAL_ERR,
+    cargo, change_versions, info, read_config, ChangeData, ChangeOpt, Error, GitOpt, Pkg, Result,
+    WorkspaceConfig, INTERNAL_ERR,
 };
 
 use cargo_metadata::Metadata;
-use clap::{ArgEnum, ArgSettings, Parser};
+use clap::{ArgEnum, Parser};
 use dialoguer::{theme::ColorfulTheme, Confirm, Input, Select};
 use oclif::{
     console::Style,
@@ -52,11 +53,7 @@ pub struct VersionOpt {
     pub custom: Option<Version>,
 
     /// Specify prerelease identifier
-    #[clap(
-        long,
-        value_name = "identifier",
-        setting(ArgSettings::ForbidEmptyValues)
-    )]
+    #[clap(long, value_name = "identifier", forbid_empty_values(true))]
     pub pre_id: Option<String>,
 
     #[clap(flatten)]
@@ -80,7 +77,8 @@ pub struct VersionOpt {
 
 impl VersionOpt {
     pub fn do_versioning(&self, metadata: &Metadata) -> Result<Map<String, Version>> {
-        let branch = self.git.validate(&metadata.workspace_root)?;
+        let config: WorkspaceConfig = read_config(&metadata.workspace_metadata)?;
+        let branch = self.git.validate(&metadata.workspace_root, &config)?;
 
         let change_data = ChangeData::new(metadata, &self.change)?;
 
@@ -102,7 +100,7 @@ impl VersionOpt {
         let mut new_versions = vec![];
 
         while !changed_p.is_empty() {
-            self.get_new_versions(&metadata, changed_p, &mut new_version, &mut new_versions)?;
+            self.get_new_versions(metadata, changed_p, &mut new_version, &mut new_versions)?;
 
             let pkgs = unchanged_p.into_iter().partition::<Vec<_>, _>(|p| {
                 let pkg = metadata
@@ -150,7 +148,7 @@ impl VersionOpt {
             )?;
         }
 
-        for (pkg, _) in &new_versions {
+        for pkg in new_versions.keys() {
             let output = cargo(&metadata.workspace_root, &["update", "-p", pkg], &[])?;
 
             if output.1.contains("error:") {
@@ -163,6 +161,7 @@ impl VersionOpt {
             &new_version,
             &new_versions,
             branch,
+            &config,
         )?;
 
         Ok(new_versions)
@@ -203,7 +202,7 @@ impl VersionOpt {
                 new_versions.push((
                     p.name.to_string(),
                     new_version.as_ref().expect(INTERNAL_ERR).clone(),
-                    cur_version.clone(),
+                    p.version.clone(),
                 ));
             }
         }
@@ -279,7 +278,7 @@ impl VersionOpt {
         };
 
         let new_version = if selected == 6 {
-            let custom = custom_pre(&cur_version);
+            let custom = custom_pre(cur_version);
 
             let preid = if let Some(preid) = &self.pre_id {
                 preid.clone()
@@ -293,7 +292,7 @@ impl VersionOpt {
                     .interact_on(&TERM_ERR)?
             };
 
-            inc_preid(&cur_version, Identifier::AlphaNumeric(preid))
+            inc_preid(cur_version, Identifier::AlphaNumeric(preid))
         } else if selected == 7 {
             if let Some(version) = &self.custom {
                 version.clone()
@@ -343,7 +342,7 @@ fn inc_preid(cur_version: &Version, preid: Identifier) -> Version {
             Identifier::AlphaNumeric(id) => {
                 version.pre = vec![preid.clone()];
 
-                if preid.to_string() == id.to_string() {
+                if preid.to_string() == *id {
                     match cur_version.pre.get(1) {
                         Some(Identifier::Numeric(n)) => {
                             version.pre.push(Identifier::Numeric(n + 1))
@@ -358,14 +357,12 @@ fn inc_preid(cur_version: &Version, preid: Identifier) -> Version {
                 if preid.to_string() == n.to_string() {
                     version.pre = cur_version.pre.clone();
 
-                    if let Some(Identifier::Numeric(n)) = version.pre.iter_mut().rfind(|x| {
-                        if let Identifier::Numeric(_) = x {
-                            true
-                        } else {
-                            false
-                        }
-                    }) {
-                        *n = *n + 1;
+                    if let Some(Identifier::Numeric(n)) = version
+                        .pre
+                        .iter_mut()
+                        .rfind(|x| matches!(x, Identifier::Numeric(_)))
+                    {
+                        *n += 1;
                     }
                 } else {
                     version.pre = vec![preid, Identifier::Numeric(0)];
